@@ -28,7 +28,7 @@ prefix="/usr/local"
 if test -f "${prefix}/share/kea/scripts/admin-utils.sh"; then
     . "${prefix}/share/kea/scripts/admin-utils.sh"
 else
-    . "/usr/obj/ports/kea-2.4.0-postgresql/kea-2.4.0/src/bin/admin/admin-utils.sh"
+    . "/pobj/kea-2.4.1-postgresql/kea-2.4.1/src/bin/admin/admin-utils.sh"
 fi
 
 # First argument must be the expected schema version <major>.<minor>
